package com.sun.xml.internal.ws.model;

import com.sun.xml.internal.ws.api.model.SEIModel;
import java.util.List;
import javax.xml.ws.Holder;
import com.sun.xml.internal.bind.api.Bridge;
import com.sun.xml.internal.ws.spi.db.WrapperComposite;
import com.sun.xml.internal.ws.spi.db.RepeatedElementBridge;
import com.sun.xml.internal.ws.spi.db.XMLBridge;
import com.sun.xml.internal.ws.api.model.JavaMethod;
import javax.xml.namespace.QName;
import com.sun.xml.internal.ws.spi.db.TypeInfo;
import com.sun.xml.internal.bind.api.TypeReference;
import javax.jws.WebParam;
import com.sun.xml.internal.ws.api.model.ParameterBinding;
import com.sun.xml.internal.ws.api.model.Parameter;

public class ParameterImpl implements Parameter
{
    private ParameterBinding binding;
    private ParameterBinding outBinding;
    private String partName;
    private final int index;
    private final WebParam.Mode mode;
    @Deprecated
    private TypeReference typeReference;
    private TypeInfo typeInfo;
    private QName name;
    private final JavaMethodImpl parent;
    WrapperParameter wrapper;
    TypeInfo itemTypeInfo;
    
    public ParameterImpl(JavaMethodImpl parent, TypeInfo type, WebParam.Mode mode, int index) {
        assert type != null;
        this.typeInfo = type;
        this.name = type.tagName;
        this.mode = mode;
        this.index = index;
        this.parent = parent;
        return;
    }
    
    public AbstractSEIModelImpl getOwner() {
        return this.parent.owner;
    }
    
    public JavaMethod getParent() {
        return this.parent;
    }
    
    public QName getName() {
        return this.name;
    }
    
    public XMLBridge getXMLBridge() {
        return this.getOwner().getXMLBridge(this.typeInfo);
    }
    
    public XMLBridge getInlinedRepeatedElementBridge() {
        TypeInfo itemType = this.getItemType();
        if (itemType != null) {
            XMLBridge xb = this.getOwner().getXMLBridge(itemType);
            if (xb != null) {
                return new RepeatedElementBridge(this.typeInfo, xb);
            }
        }
        return null;
    }
    
    public TypeInfo getItemType() {
        if (this.itemTypeInfo != null) {
            return this.itemTypeInfo;
        }
        if (this.parent.getBinding().isRpcLit() || this.wrapper == null) {
            return null;
        }
        if (!WrapperComposite.class.equals(this.wrapper.getTypeInfo().type)) {
            return null;
        }
        if (!this.getBinding().isBody()) {
            return null;
        }
        return this.itemTypeInfo = this.typeInfo.getItemType();
    }
    
    @Deprecated
    public Bridge getBridge() {
        return this.getOwner().getBridge(this.typeReference);
    }
    
    @Deprecated
    protected Bridge getBridge(TypeReference typeRef) {
        return this.getOwner().getBridge(typeRef);
    }
    
    @Deprecated
    public TypeReference getTypeReference() {
        return this.typeReference;
    }
    
    public TypeInfo getTypeInfo() {
        return this.typeInfo;
    }
    
    @Deprecated
    void setTypeReference(TypeReference type) {
        this.typeReference = type;
        this.name = type.tagName;
    }
    
    public WebParam.Mode getMode() {
        return this.mode;
    }
    
    public int getIndex() {
        return this.index;
    }
    
    public boolean isWrapperStyle() {
        return false;
    }
    
    public boolean isReturnValue() {
        return this.index == -1;
    }
    
    public ParameterBinding getBinding() {
        if (this.binding == null) {
            return ParameterBinding.BODY;
        }
        return this.binding;
    }
    
    public void setBinding(ParameterBinding binding) {
        this.binding = binding;
    }
    
    public void setInBinding(ParameterBinding binding) {
        this.binding = binding;
    }
    
    public void setOutBinding(ParameterBinding binding) {
        this.outBinding = binding;
    }
    
    public ParameterBinding getInBinding() {
        return this.binding;
    }
    
    public ParameterBinding getOutBinding() {
        if (this.outBinding == null) {
            return this.binding;
        }
        return this.outBinding;
    }
    
    public boolean isIN() {
        return this.mode == WebParam.Mode.IN;
    }
    
    public boolean isOUT() {
        return this.mode == WebParam.Mode.OUT;
    }
    
    public boolean isINOUT() {
        return this.mode == WebParam.Mode.INOUT;
    }
    
    public boolean isResponse() {
        return this.index == -1;
    }
    
    public Object getHolderValue(Object obj) {
        if (obj != null && obj instanceof Holder) {
            return ((Holder)obj).value;
        }
        return obj;
    }
    
    public String getPartName() {
        if (this.partName == null) {
            return this.name.getLocalPart();
        }
        return this.partName;
    }
    
    public void setPartName(String partName) {
        this.partName = partName;
    }
    
    void fillTypes(List<TypeInfo> types) {
        TypeInfo itemType = this.getItemType();
        types.add((itemType != null) ? itemType : this.getTypeInfo());
    }
}
